@@ -1,4 +1,4 @@
-# Modified: 14 April 2016 SDH
+# Modified: 23 May 2016 SDH
 
 summBg <- function(
   vol,
@@ -61,6 +61,12 @@ summBg <- function(
     stop('norm.name ', deparse(substitute(norm.name)), ' not found in the column names of ', deparse(substitute(setup)), '.')
   }
 
+  # Check for case when 'when' argument > all times
+  if((is.numeric(when) | is.integer(when)) && all(when > vol[, time.name])) {
+    stop('when argument (', when, ') is > all times in data.')
+
+  }
+
   # Add other checks here
 
   # Trim setup based on ids and check again for inoc.name and norm.name~~~~~~~~~~~~~~~~~~~
